@@ -82,11 +82,15 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <time.h>
 #ifndef WIN32
 #include <sys/time.h>
 #endif
-#include <sys/stat.h>
+#ifdef WIN32
+#include <wincrypt.h>
+#endif
 
 /* OpenSSL headers */
 #include <openssl/ssl.h>
@@ -497,6 +501,9 @@ typedef enum {
     SSL_RSSRC_BUILTIN = 1,
     SSL_RSSRC_FILE    = 2,
     SSL_RSSRC_EXEC    = 3
+#if SSL_LIBRARY_VERSION >= 0x00905100
+   ,SSL_RSSRC_EGD     = 4
+#endif
 } ssl_rssrc_t;
 typedef struct {
     ssl_rsctx_t  nCtx;
@@ -780,7 +787,7 @@ void         ssl_io_suck(request_rec *, SSL *);
 #endif
 
 /*  PRNG  */
-int          ssl_rand_seed(server_rec *, pool *, ssl_rsctx_t);
+int          ssl_rand_seed(server_rec *, pool *, ssl_rsctx_t, char *);
 
 /*  Extensions  */
 void         ssl_ext_register(void);
